@@ -58,6 +58,10 @@ local function save(filename)
   end
 end
 
+local function save_existing(doc)
+  if doc.filename then doc:save() end
+end
+
 local function cut_or_copy(delete)
   local full_text = ""
   local text = ""
@@ -94,6 +98,13 @@ local function cut_or_copy(delete)
   system.set_clipboard(full_text)
 end
 
+local function set_primary_selection(doc)
+  -- Doesn't work on Windows, so avoid spending time getting the text
+  if PLATFORM ~= "Windows" then
+    system.set_primary_selection(doc:get_selection_text())
+  end
+end
+
 local function split_cursor(dv, direction)
   local new_cursors = {}
   local dv_translate = direction < 0
@@ -126,6 +137,13 @@ local function set_cursor(dv, x, y, snap_type)
   core.blink_reset()
 end
 
+local function set_encoding(doc, charset)
+  doc.encoding = charset
+  if doc.bom then
+    doc.bom = encoding.get_charset_bom(charset)
+  end
+end
+
 local function line_comment(comment, line1, col1, line2, col2)
   local start_comment = (type(comment) == 'table' and comment[1] or comment) .. " "
   local end_comment = (type(comment) == 'table' and " " .. comment[2])
@@ -298,6 +316,15 @@ local commands = {
     end
   end,
 
+  ["doc:paste-primary-selection"] = function(dv, x, y)
+    if type(x) == "number" and type(y) == "number" then
+      set_cursor(dv, x, y, "set")
+      -- Workaround to avoid that a middle mouse drag starts selecting
+      dv.mouse_selecting = nil
+    end
+    dv.doc:text_input(system.get_primary_selection() or "")
+  end,
+
   ["doc:newline"] = function(dv)
     for idx, line, col in dv.doc:get_selections(false, true) do
       local indent = dv.doc.lines[line]:match("^[\t ]*")
@@ -354,6 +381,7 @@ local commands = {
 
   ["doc:select-all"] = function(dv)
     dv.doc:set_selection(1, 1, math.huge, math.huge)
+    set_primary_selection(dv.doc)
     -- avoid triggering DocView:scroll_to_make_visible
     dv.last_line1 = 1
     dv.last_col1 = 1
@@ -366,6 +394,7 @@ local commands = {
       append_line_if_last_line(line2)
       dv.doc:set_selections(idx, line2 + 1, 1, line1, 1)
     end
+    set_primary_selection(dv.doc)
   end,
 
   ["doc:select-word"] = function(dv)
@@ -374,6 +403,7 @@ local commands = {
       local line2, col2 = translate.end_of_word(dv.doc, line1, col1)
       dv.doc:set_selections(idx, line2, col2, line1, col1)
     end
+    set_primary_selection(dv.doc)
   end,
 
   ["doc:join-lines"] = function(dv)
@@ -555,24 +585,14 @@ local commands = {
 
   ["doc:change-encoding"] = function(dv)
     encodings.select_encoding("Select Output Encoding", function(charset)
-      dv.doc.encoding = charset
-      if charset ~= "UTF-8" and charset ~= "ASCII" then
-        dv.doc.convert = true
-      else
-        dv.doc.convert = false
-      end
-      dv.doc:save()
+      set_encoding(dv.doc, charset)
+      save_existing(dv.doc)
     end)
   end,
 
   ["doc:reload-with-encoding"] = function(dv)
     encodings.select_encoding("Reload With Encoding", function(charset)
-      dv.doc.encoding = charset
-      if charset ~= "UTF-8" and charset ~= "ASCII" then
-        dv.doc.convert = true
-      else
-        dv.doc.convert = false
-      end
+      set_encoding(dv.doc, charset)
       dv.doc:reload()
     end)
   end,
@@ -655,6 +675,7 @@ local commands = {
     local line2, col2 = dv:resolve_screen_position(x, y)
     dv.mouse_selecting = { line1, col1, nil }
     dv.doc:set_selection(line2, col2, line1, col1)
+    set_primary_selection(dv.doc)
   end,
 
   ["doc:create-cursor-previous-line"] = function(dv)
@@ -703,6 +724,28 @@ end, {
   end
 })
 
+command.add(function()
+  if not core.active_view:extends(DocView) then return false end
+  local doc = core.active_view.doc
+  local bom = encoding.get_charset_bom(doc.encoding or "none")
+  return  bom ~= nil, doc, bom
+end, {
+  ["doc:disable-bom"] = function(doc)
+    doc.bom = nil
+    save_existing(doc)
+  end,
+
+  ["doc:enable-bom"] = function(doc, bom)
+    doc.bom = bom
+    save_existing(doc)
+  end,
+
+  ["doc:toggle-bom"] = function(doc, bom)
+    if doc.bom then doc.bom = nil else doc.bom = bom end
+    save_existing(doc)
+  end,
+})
+
 local translations = {
   ["previous-char"] = translate,
   ["next-char"] = translate,
@@ -724,9 +767,16 @@ local translations = {
 }
 
 for name, obj in pairs(translations) do
-  commands["doc:move-to-" .. name] = function(dv) dv.doc:move_to(obj[name:gsub("-", "_")], dv) end
-  commands["doc:select-to-" .. name] = function(dv) dv.doc:select_to(obj[name:gsub("-", "_")], dv) end
-  commands["doc:delete-to-" .. name] = function(dv) dv.doc:delete_to(obj[name:gsub("-", "_")], dv) end
+  commands["doc:move-to-" .. name] = function(dv)
+    dv.doc:move_to(obj[name:gsub("-", "_")], dv)
+  end
+  commands["doc:select-to-" .. name] = function(dv)
+    dv.doc:select_to(obj[name:gsub("-", "_")], dv)
+    set_primary_selection(dv.doc)
+  end
+  commands["doc:delete-to-" .. name] = function(dv)
+    dv.doc:delete_to(obj[name:gsub("-", "_")], dv)
+  end
 end
 
 commands["doc:move-to-previous-char"] = function(dv)
