@@ -1,6 +1,6 @@
 local Object = require "core.object"
-local Highlighter = require "core.doc.highlighter"
-local translate = require "core.doc.translate"
+local Highlighter = require ".highlighter"
+local translate = require ".translate"
 local core = require "core"
 local syntax = require "core.syntax"
 local config = require "core.config"
@@ -23,8 +23,9 @@ end
 function Doc:new(filename, abs_filename, new_file)
   self.new_file = new_file
   self.encoding = nil
-  self.convert = false
+  self.bom = nil
   self.binary = false
+  self.cache = {}
   self:reset()
   if filename then
     self:set_filename(filename, abs_filename)
@@ -84,25 +85,33 @@ function Doc:set_filename(filename, abs_filename)
 end
 
 
+function Doc:needs_encoding_conversion()
+  local charset = self.encoding
+  if charset and charset ~= "UTF-8" and charset ~= "ASCII" then
+    return true
+  end
+  return false
+end
+
+
 function Doc:load(filename)
   if not self.encoding then
     local errmsg
-    self.encoding, errmsg = encoding.detect(filename);
+    self.encoding, self.bom, errmsg = encoding.detect(filename);
     if not self.encoding then
       core.error("%s", errmsg)
       self.encoding = "UTF-8"
     end
+  elseif self.bom then
+    self.bom = encoding.get_charset_bom(self.encoding)
   end
-  self.convert = false
-  if self.encoding ~= "UTF-8" and self.encoding ~= "ASCII" then
-    self.convert = true
-  end
+  local convert = self:needs_encoding_conversion()
   local fp = assert( io.open(filename, "rb") )
   self:reset()
   self.lines = {}
   self.clean_lines = {}
   local i = 1
-  if self.convert then
+  if convert then
     local content = fp:read("*a");
     content = assert(encoding.convert("UTF-8", self.encoding, content, {
       strict = false,
@@ -182,8 +191,10 @@ function Doc:save(filename, abs_filename)
   end
   local fp
   local output = ""
-  if not self.convert then
+  local convert = self:needs_encoding_conversion()
+  if not convert then
     fp = open_for_writing(abs_filename)
+    if self.bom then fp:write(self.bom) end
     for _, line in ipairs(self.lines) do
       if self.crlf then line = line:gsub("\n", "\r\n") end
       fp:write(line)
@@ -193,15 +204,15 @@ function Doc:save(filename, abs_filename)
       if self.crlf then output = output:gsub("\n", "\r\n") end
   end
   local conversion_error = false
-  if self.convert then
+  if convert then
     local errmsg
     output, errmsg = encoding.convert(self.encoding, "UTF-8", output, {
-      strict = true,
-      handle_to_bom = true
+      strict = true
     })
     if output then
       fp = open_for_writing(abs_filename)
-      fp:write(encoding.get_charset_bom(self.encoding) .. output)
+      if self.bom then fp:write(self.bom) end
+      fp:write(output)
       fp:close()
     else
       conversion_error = true
@@ -353,7 +364,8 @@ function Doc:set_selection(line1, col1, line2, col2, swap)
 end
 
 function Doc:merge_cursors(idx)
-  for i = (idx or (#self.selections - 3)), (idx or 5), -4 do
+  local table_index = idx and (idx - 1) * 4 + 1
+  for i = (table_index or (#self.selections - 3)), (table_index or 5), -4 do
     for j = 1, i - 4, 4 do
       if self.selections[i] == self.selections[j] and
         self.selections[i+1] == self.selections[j+1] then
@@ -435,18 +447,29 @@ function Doc:position_offset(line, col, ...)
 end
 
 
-function Doc:get_text(line1, col1, line2, col2)
+---Returns the content of the doc between two positions. </br>
+---The positions will be sanitized and sorted. </br>
+---The character at the "end" position is not included by default.
+---@see core.doc.sanitize_position
+---@param line1 integer
+---@param col1 integer
+---@param line2 integer
+---@param col2 integer
+---@param inclusive boolean? Whether or not to return the character at the last position
+---@return string
+function Doc:get_text(line1, col1, line2, col2, inclusive)
   line1, col1 = self:sanitize_position(line1, col1)
   line2, col2 = self:sanitize_position(line2, col2)
   line1, col1, line2, col2 = sort_positions(line1, col1, line2, col2)
+  local col2_offset = inclusive and 0 or 1
   if line1 == line2 then
-    return self.lines[line1]:sub(col1, col2 - 1)
+    return self.lines[line1]:sub(col1, col2 - col2_offset)
   end
   local lines = { self.lines[line1]:sub(col1) }
   for i = line1 + 1, line2 - 1 do
     table.insert(lines, self.lines[i])
   end
-  table.insert(lines, self.lines[line2]:sub(1, col2 - 1))
+  table.insert(lines, self.lines[line2]:sub(1, col2 - col2_offset))
   return table.concat(lines)
 end
 
@@ -510,6 +533,20 @@ local function update_clean_lines(self, line1, line2)
 end
 
 
+function Doc:clear_cache(l, n)
+  for _, cache in pairs(self.cache) do
+    local lines = l + n
+    for ln=l-1, lines do
+      local line = ln + 1
+      if cache[line] then
+        cache[line] = nil
+      end
+      if line == lines then break end
+    end
+  end
+end
+
+
 function Doc:raw_insert(line, col, text, undo_stack, time)
   -- split text into lines and merge with line at insertion point
   local lines = split_lines(text)
@@ -542,6 +579,7 @@ function Doc:raw_insert(line, col, text, undo_stack, time)
 
   -- update highlighter and assure selection is in bounds
   self.highlighter:insert_notify(line, #lines - 1)
+  self:clear_cache(line, #lines - 1)
   self:sanitize_selection()
 end
 
@@ -603,6 +641,7 @@ function Doc:raw_remove(line1, col1, line2, col2, undo_stack, time)
 
   -- update highlighter and assure selection is in bounds
   self.highlighter:remove_notify(line1, line_removal)
+  self:clear_cache(line1, line_removal)
   self:sanitize_selection()
 end
 
@@ -648,7 +687,7 @@ function Doc:text_input(text, idx)
     if self.overwrite
     and (line1 == line2 and col1 == col2)
     and col1 < #self:get_utf8_line(line1)
-    and text:ulen() == 1 then
+    and text:ulen(nil, nil, true) == 1 then
       self:remove(line1, col1, translate.next_char(self, line1, col1))
     end
 
@@ -800,6 +839,10 @@ end
 
 -- For plugins to get notified when a document is closed
 function Doc:on_close()
+  -- this shouldn't be needed but we do it to better hint the gc to collect
+  self.highlighter.doc = nil
+  self.highlighter.lines = nil
+
   core.log_quiet("Closed doc \"%s\"", self:get_name())
 end
 
