@@ -1,8 +1,10 @@
 #include <SDL.h>
 #include <string.h>
 #include <stdbool.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
+#include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "api.h"
@@ -807,9 +809,10 @@ static int f_chdir(lua_State *L) {
 static int f_getcwd(lua_State* L) {
   #ifdef _WIN32
     wchar_t buffer[MAX_PATH];
-    if (!_wgetcwd(buffer, sizeof(buffer)))
+    if (!_wgetcwd(buffer, MAX_PATH))
       return luaL_error(L, "error getcwd: %s", strerror(errno));
     char *utf8_buffer = utfconv_wctoutf8(buffer);
+    if (utf8_buffer == NULL) { return luaL_error(L, UTFCONV_ERROR_INVALID_CONVERSION ); }
     lua_pushstring(L, utf8_buffer);
     free(utf8_buffer);
   #else
@@ -958,7 +961,7 @@ static int f_get_file_info(lua_State *L) {
   lua_pushstring(L, data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? "dir" : "file");
   lua_setfield(L, -2, "type");
 
-  lua_pushboolean(L, data.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT);
+  lua_pushboolean(L, data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY && data.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT);
   lua_setfield(L, -2, "symlink");
 #else
   struct stat s;
