@@ -31,7 +31,8 @@ static void setup_renderer(RenWindow *ren, int w, int h) {
 #endif
 
 
-void renwin_init_surface(UNUSED RenWindow *ren) {
+void renwin_init_surface(RenWindow *ren) {
+  ren->scale_x = ren->scale_y = 1;
 #ifdef PRAGTICAL_USE_SDL_RENDERER
   if (ren->rensurface.surface) {
     SDL_FreeSurface(ren->rensurface.surface);
@@ -113,6 +114,16 @@ void renwin_resize_surface(UNUSED RenWindow *ren) {
 #endif
 }
 
+void renwin_update_scale(RenWindow *ren) {
+#ifndef PRAGTICAL_USE_SDL_RENDERER
+  SDL_Surface *surface = SDL_GetWindowSurface(ren->window);
+  int window_w = surface->w, window_h = surface->h;
+  SDL_GetWindowSize(ren->window, &window_w, &window_h);
+  ren->scale_x = (float)surface->w / window_w;
+  ren->scale_y = (float)surface->h / window_h;
+#endif
+}
+
 void renwin_show_window(RenWindow *ren) {
   SDL_ShowWindow(ren->window);
 }
